from flask import jsonify, request
from flask_cors import cross_origin
from flask_expects_json import expects_json
from flask_jwt_extended import get_jwt, jwt_required

from areas import api_v1
from helpers import KratosApi
from helpers.auth_guard import admin_required, kratos_webhook

from .validation import schema, schema_multiple, schema_multi_edit, schema_recovery_complete
from .user_service import UserService


@api_v1.route("/users", methods=["GET"])
@jwt_required()
@cross_origin()
@admin_required()
def get_users():
    res = UserService.get_users()
    return jsonify(res)


@api_v1.route("/users/<string:id>", methods=["GET"])
@jwt_required()
@cross_origin()
@admin_required()
def get_user(id):
    res = UserService.get_user(id)
    return jsonify(res)

@api_v1.route("/users/<string:id>/recovery", methods=["POST"])
@jwt_required()
@cross_origin()
@admin_required()
def get_user_recovery(id):
    res = UserService.create_recovery_link(id)
    return jsonify(res)

@api_v1.route("/users/<string:id>/reset_2fa", methods=["POST"])
@jwt_required()
@cross_origin()
@admin_required()
def reset_2fa(id):
    res = UserService.reset_2fa(id)
    return jsonify(res)

# This is supposed to be called by Kratos as a webhook after a user has
# successfully recovered their account.
@api_v1.route("/users/recovery_complete", methods=["POST"])
@expects_json(schema_recovery_complete)
@kratos_webhook()
def recovery_complete():
    data = request.get_json()
    try:
        UserService.recovery_complete(data["user_id"])
    except Exception as e:
        current_app.logger.warn(f"Exception in /users/recovery_complete: {e}")
        raise
    return jsonify(message="Updated last recovery time.")

# This is supposed to be called by Kratos as a webhook after a user has
# successfully logged in to their account.
@api_v1.route("/users/login_complete", methods=["POST"])
@expects_json(schema_recovery_complete)
@kratos_webhook()
def login_complete():
    data = request.get_json()
    try:
        UserService.login_complete(data["user_id"])
    except Exception as e:
        current_app.logger.warn(f"Exception in /users/login_complete: {e}")
        raise
    return jsonify(message="Updated last login time.")

@api_v1.route("/users", methods=["POST"])
@jwt_required()
@cross_origin()
@expects_json(schema)
@admin_required()
def post_user():
    data = request.get_json()
    res = UserService.post_user(data)
    return jsonify(res)


@api_v1.route("/users/<string:id>", methods=["PUT"])
@jwt_required()
@cross_origin()
@expects_json(schema)
@admin_required()
def put_user(id):
    data = request.get_json()
    res = UserService.put_user(id, data)
    return jsonify(res)


@api_v1.route("/users/<string:id>", methods=["DELETE"])
@jwt_required()
@cross_origin()
@admin_required()
def delete_user(id):
    res = KratosApi.delete("/identities/{}".format(id))
    if res.status_code == 204:
        UserService.delete_user(id)
        return jsonify(), res.status_code
    return jsonify(res.json()), res.status_code


@api_v1.route("/users-batch", methods=["POST"])
@jwt_required()
@cross_origin()
@expects_json(schema_multiple)
@admin_required()
def post_multiple_users():
    """Expects an array of user JSON schema in request body."""
    data = request.get_json()
    res = UserService.post_multiple_users(data)
    return jsonify(res)


# multi-user editing of app roles
@api_v1.route("/users-multi-edit", methods=["PUT"])
@jwt_required()
@cross_origin()
@expects_json(schema_multi_edit)
@admin_required()
def put_multiple_users():
    data = request.get_json()
    user_id = __get_user_id_from_jwt()
    res = UserService.put_multiple_users(user_id, data)
    return jsonify(res)

@api_v1.route("/me", methods=["GET"])
@jwt_required()
@cross_origin()
def get_personal_info():
    user_id = __get_user_id_from_jwt()
    res = UserService.get_user(user_id)
    return jsonify(res)


@api_v1.route("/me", methods=["PUT"])
@jwt_required()
@cross_origin()
@expects_json(schema)
def update_personal_info():
    data = request.get_json()
    user_id = __get_user_id_from_jwt()
    res = UserService.put_user(user_id, user_id, data)
    return jsonify(res)


def __get_user_id_from_jwt():
    claims = get_jwt()
    return claims["user_id"]
