from database import db
from areas.apps.models import App, OAuthClientApp
import helpers.kubernetes as k8s

import logging
import yaml

# Read in two configmaps from the cluster, which specify which apps should be
# present in the database.
def populate_apps():
    logging.info("cluster_config: populating apps")
    database_apps = {}
    for app in App.query.all():
        slug = app.slug
        database_apps[slug] = app
        logging.debug(f"database app: {slug}")
    _populate_apps_from(database_apps, "stackspin-apps")
    _populate_apps_from(database_apps, "stackspin-apps-custom")

# Read a list of apps from a configmap. Check if they are already present in
# the database, and if not, add missing ones there. Properties `name`,
# `external` and `url` can be specified in yaml format in the configmap value
# contents.
def _populate_apps_from(database_apps, configmap_name):
    cm_apps = k8s.get_kubernetes_config_map_data(configmap_name, "flux-system")
    if cm_apps is None:
        logging.info(f"Could not find configmap '{configmap_name}' in namespace 'flux-system'; ignoring.")
    else:
        for app_slug, app_data in cm_apps.items():
            logging.debug(f"configmap app: {app_slug}")
            if app_slug in database_apps:
                logging.debug(f"  already present in database")
            else:
                logging.debug(f"  not present in database, adding!")
                data = yaml.safe_load(app_data)
                name = data["name"]
                logging.debug(f"  name: {name}")
                external = data.get("external", False)
                logging.debug(f"  type external: {type(external)}")
                logging.debug(f"  external: {external}")
                url = data.get("url", None)
                logging.debug(f"  url: {url}")
                new_app = App(slug=app_slug, name=name, external=external, url=url)
                db.session.add(new_app)
        db.session.commit()

# Read in two configmaps from the cluster, which specify which oauthclients
# should be present in the database.
def populate_oauthclients():
    logging.info("cluster_config: populating oauthclients")
    database_oauthclients = {}
    for client in OAuthClientApp.query.all():
        id = client.oauthclient_id
        database_oauthclients[id] = client
        logging.debug(f"database oauthclient: {id}")
    _populate_oauthclients_from(database_oauthclients, "stackspin-oauthclients")
    _populate_oauthclients_from(database_oauthclients, "stackspin-oauthclients-custom")

# Read a list of oauthclients from a configmap. Check if they are already
# present in the database, and if not, add missing ones there. The value of the
# mapping is taken to be the slug of the app the oauthclient belongs to.
def _populate_oauthclients_from(database_oauthclients, configmap_name):
    cm_oauthclients = k8s.get_kubernetes_config_map_data(configmap_name, "flux-system")
    if cm_oauthclients is None:
        logging.info(f"Could not find configmap '{configmap_name}' in namespace 'flux-system'; ignoring.")
    else:
        for client_id, client_app in cm_oauthclients.items():
            logging.debug(f"configmap oauthclient: {client_id}")
            if client_id in database_oauthclients:
                logging.debug(f"  already present in database")
            else:
                logging.debug(f"  not present in database, adding!")
                # Take the value of the configmap mapping (`client_app`) and
                # interpret it as the slug of the app that this oauthclient
                # belongs to.
                app = App.query.filter_by(slug=client_app).first()
                if not app:
                    logging.error(f"  could not find app with slug {client_app}")
                    continue
                new_client = OAuthClientApp(oauthclient_id=client_id, app_id=app.id)
                logging.debug(f"  new oauth client: {new_client}")
                db.session.add(new_client)
        db.session.commit()
